local F, C, L = unpack(select(2, ...))
local MISC, cfg = F:RegisterModule('Misc'), C.general


local _G = getfenv(0)
local tostring, tonumber, pairs, select, random, strsplit, strmatch = tostring, tonumber, pairs, select, math.random, string.split, string.match


function MISC:OnLogin()
	self:AutoDismount()
	self:ColorPicker()
	self:EnhancedMenu()
	self:FasterDelete()
	self:ErrorFrame()
	self:HelmCloak()
	self:ItemLevel()
	self:Marker()
	self:MailButton()
	self:PVPSound()
	self:ReadyCheck()
	self:ReanchorTicketStatusFrame()
	self:Reputation()
	self:TradeTargetInfo()
	self:AddSharedMedia()
	self:QuickBuy()

	-- Fix blizz error
	MAIN_MENU_MICRO_ALERT_PRIORITY = MAIN_MENU_MICRO_ALERT_PRIORITY or {}
end



-- Easily hide helm and cloak
function MISC:HelmCloak()
	if not C.general.helmCloak then return end

	local helm = CreateFrame("CheckButton", "FreeUI_HelmCheckBox", PaperDollFrame, "OptionsCheckButtonTemplate")
	helm:SetSize(22, 22)
	helm:SetPoint("LEFT", CharacterModelFrame, "BOTTOMLEFT", 0, 44)
	helm:SetScript("OnClick", function() ShowHelm(not ShowingHelm()) end)
	helm:SetScript("OnEvent", function() helm:SetChecked(ShowingHelm()) end)
	helm:RegisterEvent("UNIT_MODEL_CHANGED")
	helm:SetToplevel(true)
	helm.text = F.CreateFS(helm, (C.isCNClient and {C.font.normal, 11}) or 'pixel', L['MISC_SHOW_HELM'], nil, true, 'LEFT', 22, 1)

	local cloak = CreateFrame("CheckButton", "FreeUI_CloakCheckBox", PaperDollFrame, "OptionsCheckButtonTemplate")
	cloak:SetSize(22, 22)
	cloak:SetPoint("LEFT", CharacterModelFrame, "BOTTOMLEFT", 0, 24)
	cloak:SetScript("OnClick", function() ShowCloak(not ShowingCloak()) end)
	cloak:SetScript("OnEvent", function() cloak:SetChecked(ShowingCloak()) end)
	cloak:RegisterEvent("UNIT_MODEL_CHANGED")
	cloak:SetToplevel(true)
	cloak.text = F.CreateFS(cloak, (C.isCNClient and {C.font.normal, 11}) or 'pixel', L['MISC_SHOW_CLOAK'], nil, true, 'LEFT', 22, 1)

	helm:SetChecked(ShowingHelm())
	cloak:SetChecked(ShowingCloak())
	helm:SetFrameLevel(31)
	cloak:SetFrameLevel(31)

	F.ReskinCheck(helm)
	F.ReskinCheck(cloak)
end

-- Instant delete
function MISC:FasterDelete()
	hooksecurefunc(StaticPopupDialogs['DELETE_GOOD_ITEM'], 'OnShow', function(self)
		self.editBox:SetText(DELETE_ITEM_CONFIRM_STRING)
	end)
end

-- Ready check in master sound
function MISC:ReadyCheck()
	local f = CreateFrame('Frame')
	f:RegisterEvent('UPDATE_BATTLEFIELD_STATUS')

	f:RegisterEvent('RESURRECT_REQUEST')
	f:RegisterEvent('READY_CHECK')
	f:SetScript('OnEvent', function(self, event)
		if event == 'UPDATE_BATTLEFIELD_STATUS' then
			for i = 1, GetMaxBattlefieldID() do
				local status = GetBattlefieldStatus(i)
				if status == 'confirm' then
					PlaySound(SOUNDKIT.PVP_THROUGH_QUEUE, 'Master')
					break
				end
				i = i + 1
			end
		elseif event == 'PET_BATTLE_QUEUE_PROPOSE_MATCH' then
			PlaySound(SOUNDKIT.PVP_THROUGH_QUEUE, 'Master')
		elseif event == 'LFG_PROPOSAL_SHOW' then
			PlaySound(SOUNDKIT.READY_CHECK, 'Master')
		elseif event == 'RESURRECT_REQUEST' then
			PlaySound(37, 'Master')
		elseif event == 'READY_CHECK' then
			PlaySound(SOUNDKIT.READY_CHECK, 'master')
		end
	end)
end

-- TradeFrame hook
function MISC:TradeTargetInfo()
	local infoText = F.CreateFS(TradeFrame, {C.font.normal, 14}, '', nil, nil, true)
	infoText:ClearAllPoints()
	infoText:SetPoint('TOP', TradeFrameRecipientNameText, 'BOTTOM', 0, -5)

	local function updateColor()
		local r, g, b = F.UnitColor('NPC')
		TradeFrameRecipientNameText:SetTextColor(r, g, b)

		local guid = UnitGUID('NPC')
		if not guid then return end
		local text = '|cffff0000'..L['MISC_STRANGER']
		if BNGetGameAccountInfoByGUID(guid) or C_FriendList.IsFriend(guid) then
			text = '|cffffff00'..FRIEND
		elseif IsGuildMember(guid) then
			text = '|cff00ff00'..GUILD
		end
		infoText:SetText(text)
	end
	hooksecurefunc('TradeFrame_Update', updateColor)
end

-- Reanchor TicketStatusFrame
function MISC:ReanchorTicketStatusFrame()
	hooksecurefunc(TicketStatusFrame, 'SetPoint', function(self, relF)
		if relF == 'TOPRIGHT' then
			self:ClearAllPoints()
			self:SetPoint('TOP', UIParent, 'TOP', 0, -100)
		end
	end)
end

-- Select target when click on raid units
do
	local function fixRaidGroupButton()
		for i = 1, 40 do
			local bu = _G['RaidGroupButton'..i]
			if bu and bu.unit and not bu.clickFixed then
				bu:SetAttribute('type', 'target')
				bu:SetAttribute('unit', bu.unit)

				bu.clickFixed = true
			end
		end
	end

	local function setupMisc(event, addon)
		if event == 'ADDON_LOADED' and addon == 'Blizzard_RaidUI' then
			if not InCombatLockdown() then
				fixRaidGroupButton()
			else
				F:RegisterEvent('PLAYER_REGEN_ENABLED', setupMisc)
			end
			F:UnregisterEvent(event, setupMisc)
		elseif event == 'PLAYER_REGEN_ENABLED' then
			if RaidGroupButton1 and RaidGroupButton1:GetAttribute('type') ~= 'target' then
				fixRaidGroupButton()
				F:UnregisterEvent(event, setupMisc)
			end
		end
	end

	F:RegisterEvent('ADDON_LOADED', setupMisc)
end

-- ALT+RightClick to buy a stack
function MISC:QuickBuy()
	if not cfg.quickBuy then return end
	
	local cache = {}
	local itemLink, id

	StaticPopupDialogs['BUY_STACK'] = {
		text = L['MISC_STACK_BUYING_CHECK'],
		button1 = YES,
		button2 = NO,
		OnAccept = function()
			if not itemLink then return end
			BuyMerchantItem(id, GetMerchantItemMaxStack(id))
			cache[itemLink] = true
			itemLink = nil
		end,
		hasItemFrame = 1,
		timeout = 0,
		whileDead = 1,
		hideOnEscape = true,
		preferredIndex = 5,
	}

	local _MerchantItemButton_OnModifiedClick = MerchantItemButton_OnModifiedClick
	function MerchantItemButton_OnModifiedClick(self, ...)
		if IsAltKeyDown() then
			id = self:GetID()
			itemLink = GetMerchantItemLink(id)
			if not itemLink then return end
			local name, _, quality, _, _, _, _, maxStack, _, texture = GetItemInfo(itemLink)
			if maxStack and maxStack > 1 then
				if not cache[itemLink] then
					local r, g, b = GetItemQualityColor(quality or 1)
					StaticPopup_Show('BUY_STACK', ' ', ' ', {['texture'] = texture, ['name'] = name, ['color'] = {r, g, b, 1}, ['link'] = itemLink, ['index'] = id, ['count'] = maxStack})
				else
					BuyMerchantItem(id, GetMerchantItemMaxStack(id))
				end
			end
		end

		_MerchantItemButton_OnModifiedClick(self, ...)
	end
end

-- Show BID and highlight price
do
	local function setupMisc(event, addon)
		if addon == 'Blizzard_AuctionUI' then
			hooksecurefunc('AuctionFrameBrowse_Update', function()
				local numBatchAuctions = GetNumAuctionItems('list')
				local offset = FauxScrollFrame_GetOffset(BrowseScrollFrame)
				local name, buyoutPrice, bidAmount, hasAllInfo
				for i = 1, NUM_BROWSE_TO_DISPLAY do
					local index = offset + i + (NUM_AUCTION_ITEMS_PER_PAGE * AuctionFrameBrowse.page)
					local shouldHide = index > (numBatchAuctions + (NUM_AUCTION_ITEMS_PER_PAGE * AuctionFrameBrowse.page))
					if not shouldHide then
						name, _, _, _, _, _, _, _, _, buyoutPrice, bidAmount, _, _, _, _, _, _, hasAllInfo = GetAuctionItemInfo('list', offset + i)
						if not hasAllInfo then shouldHide = true end
					end
					if not shouldHide then
						local alpha = .5
						local color = 'yellow'
						local buttonName = 'BrowseButton'..i
						local itemName = _G[buttonName..'Name']
						local moneyFrame = _G[buttonName..'MoneyFrame']
						local buyoutMoney = _G[buttonName..'BuyoutFrameMoney']
						if buyoutPrice >= 5*1e7 then color = 'red' end
						if bidAmount > 0 then
							name = name..' |cffffff00'..BID..'|r'
							alpha = 1.0
						end
						itemName:SetText(name)
						moneyFrame:SetAlpha(alpha)
						SetMoneyFrameColor(buyoutMoney:GetName(), color)
					end
				end
			end)

			F:UnregisterEvent(event, setupMisc)
		end
	end

	F:RegisterEvent('ADDON_LOADED', setupMisc)
end

-- Temporary taint fix
do
	InterfaceOptionsFrameCancel:SetScript("OnClick", function()
		InterfaceOptionsFrameOkay:Click()
	end)
end


-- make it only split stacks with shift-rightclick if the TradeSkillFrame is open
-- shift-leftclick should be reserved for the search box
local function hideSplitFrame(_, button)
	if TradeSkillFrame and TradeSkillFrame:IsShown() then
		if button == 'LeftButton' then
			StackSplitFrame:Hide()
		end
	end
end
hooksecurefunc('ContainerFrameItemButton_OnModifiedClick', hideSplitFrame)
hooksecurefunc('MerchantItemButton_OnModifiedClick', hideSplitFrame)

-- Fix blizz guild news hyperlink
do
	local function fixGuildNews(event, addon)
		if addon ~= 'Blizzard_GuildUI' then return end

		local _GuildNewsButton_OnEnter = GuildNewsButton_OnEnter
		function GuildNewsButton_OnEnter(self)
			if not (self.newsInfo and self.newsInfo.whatText) then return end
			_GuildNewsButton_OnEnter(self)
		end

		F:UnregisterEvent(event, fixGuildNews)
	end

	local function fixCommunitiesNews(event, addon)
		if addon ~= 'Blizzard_Communities' then return end

		local _CommunitiesGuildNewsButton_OnEnter = CommunitiesGuildNewsButton_OnEnter
		function CommunitiesGuildNewsButton_OnEnter(self)
			if not (self.newsInfo and self.newsInfo.whatText) then return end
			_CommunitiesGuildNewsButton_OnEnter(self)
		end

		F:UnregisterEvent(event, fixCommunitiesNews)
	end

	F:RegisterEvent('ADDON_LOADED', fixGuildNews)
	F:RegisterEvent('ADDON_LOADED', fixCommunitiesNews)
end

-- Add friend and guild invite on target menu
function MISC:MenuButton_OnClick(info)
	local name, server = UnitName(info.unit)
	if server and server ~= "" then name = name.."-"..server end

	if info.value == "name" then
		if MailFrame:IsShown() then
			MailFrameTab_OnClick(nil, 2)
			SendMailNameEditBox:SetText(name)
			SendMailNameEditBox:HighlightText()
		else
			local editBox = ChatEdit_ChooseBoxForSend()
			local hasText = (editBox:GetText() ~= "")
			ChatEdit_ActivateChat(editBox)
			editBox:Insert(name)
			if not hasText then editBox:HighlightText() end
		end
	elseif info.value == "guild" then
		GuildInvite(name)
	end
end

function MISC:MenuButton_Show(_, unit)
	if UIDROPDOWNMENU_MENU_LEVEL > 1 then return end

	if unit and (unit == "target" or string.find(unit, "party") or string.find(unit, "raid")) then
		local info = UIDropDownMenu_CreateInfo()
		info.text = MISC.MenuButtonList["name"]
		info.arg1 = {value = "name", unit = unit}
		info.func = MISC.MenuButton_OnClick
		info.notCheckable = true
		UIDropDownMenu_AddButton(info)

		if IsInGuild() and UnitIsPlayer(unit) and not UnitCanAttack("player", unit) and not UnitIsUnit("player", unit) then
			info = UIDropDownMenu_CreateInfo()
			info.text = MISC.MenuButtonList["guild"]
			info.arg1 = {value = "guild", unit = unit}
			info.func = MISC.MenuButton_OnClick
			info.notCheckable = true
			UIDropDownMenu_AddButton(info)
		end
	end
end

function MISC:EnhancedMenu()
	if not C.general.enhancedMenu then return end

	MISC.MenuButtonList = {
		["name"] = COPY_NAME,
		["guild"] = gsub(CHAT_GUILD_INVITE_SEND, HEADER_COLON, ""),
	}
	hooksecurefunc("UnitPopup_ShowMenu", MISC.MenuButton_Show)
end

function MISC:AddSharedMedia()
	local LSM = LibStub('LibSharedMedia-3.0')
	if not LSM then return end

	local cn, tw, western = LSM.LOCALE_BIT_zhCN, LSM.LOCALE_BIT_zhTW, LSM.LOCALE_BIT_western

	LSM:Register(LSM.MediaType.BACKGROUND, 'FreeUI_Background', 	C.media.bdTex)

	LSM:Register(LSM.MediaType.STATUSBAR, 'FreeUI_Statusbar', 		C.media.sbTex)

	LSM:Register(LSM.MediaType.FONT, 'FreeUI_PixelAlt', 			C.AssetsPath..'font\\supereffective.ttf', cn + tw + western)
	LSM:Register(LSM.MediaType.FONT, 'FreeUI_Pixel', 				C.AssetsPath..'font\\pixel.ttf', cn + tw + western)
	LSM:Register(LSM.MediaType.FONT, 'FreeUI_Normal', 				C.font.normal, cn + tw + western)
	LSM:Register(LSM.MediaType.FONT, 'FreeUI_Header', 				C.font.header, cn + tw + western)
	LSM:Register(LSM.MediaType.FONT, 'FreeUI_Chat', 				C.font.chat, cn + tw + western)

	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_Buzz', 				C.AssetsPath..'sound\\buzz.ogg')
	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_Buzz', 				C.AssetsPath..'sound\\buzz.ogg')
	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_Ding', 				C.AssetsPath..'sound\\ding.ogg')
	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_Execute', 			C.AssetsPath..'sound\\execute.ogg')
	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_LowHealth', 			C.AssetsPath..'sound\\lowhealth.ogg')
	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_LowMana', 			C.AssetsPath..'sound\\lowmana.ogg')
	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_Miss', 				C.AssetsPath..'sound\\miss.mp3')
	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_Proc', 				C.AssetsPath..'sound\\proc.ogg')
	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_Interrupt', 			C.AssetsPath..'sound\\interrupt.ogg')
	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_Notification', 		C.AssetsPath..'sound\\notification.mp3')
	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_Warning', 			C.AssetsPath..'sound\\warning.ogg')
	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_WhisperNormal', 		C.AssetsPath..'sound\\whisper_normal.ogg')
	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_WhisperBN', 			C.AssetsPath..'sound\\whisper_bn.ogg')
	LSM:Register(LSM.MediaType.SOUND, 'FreeUI_Forthehorde', 		C.AssetsPath..'sound\\forthehorde.mp3')
end